﻿/**
 * RadicalChatService
 * Copyright Jaromir Sivic 2009 
 */

using System;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Linq;

[WebService(Namespace = "http://radicalchatservice.cze.cz/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
//[System.Web.Script.Services.ScriptService]
public class RadicalChatService : System.Web.Services.WebService
{
    public RadicalChatService () {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }


    /**
     * !!!!!!!!
     * Please do not access variables and properties 
     * in classes "Client", "Stream", "ChatRoom" directly.
     * Use setters, getters and implemented methods
     */


    /// <summary>
    /// Always return actual ID of web service api
    /// </summary>
    /// <returns></returns>
    [WebMethod]
    public string getAPICode()
    {
        return "RadicalChat 1.0.0";
    }

    /// <summary>
    /// Return set of implemented events.
    /// If method is not implemented, then default behaviour will be used.
    /// </summary>
    /// <returns></returns>
    [WebMethod]
    public string getImplementedFunctions()
    {
        return "onApplicationStarted,onApplicationStopped,onClientWantsToConnect," +
               "onClientConnectionWasAccepted,onClientConnectionWasRejected," +
               "onClientDisconnected,onClientWantsToStartStreaming,onClientStoppedStreaming" +
               "onClientWantsToPlayStream,onClientStoppedPlayingStream," +
               "onCheckClients,onClientWantsToKickOffClients,onClientSendMessage," +
               "onUniversalCall,onChatRoomCreated,onCheckChatRooms,onChatRoomClosed";
    }

    /// <summary>
    /// This method is called when stream server and application is started
    /// </summary>
    [WebMethod(EnableSession=true)]
    public void onApplicationStarted()
    {

    }

    /// <summary>
    /// This method is called when stream server is down - it should appear
    /// just before server is restarted.
    /// </summary>
    [WebMethod]
    public void onApplicationStopped()
    {

    }

    /// <summary>
    /// When new client enter the chat, he wants to know chatroom, invitation message, etc...
    /// 
    /// What must be done:
    ///     Client must be accepted or rejected by "Accept" or "Reject" method
    ///     "client.RoomUID" must be setted, otherwise client will be rejected
    ///         if room with this roomUID does not exist, than new ChatRoom will be created
    ///         and onChatRoomCreated event will be called.
    ///     
    /// What could be done:
    ///     "nextCheck" can be setted, otherwise client will never be checked again, that means
    ///         client will never call onCheckClients
    ///     "nickname" can be setted
    ///     invitation message can be sended by "SendPrivateMessage" function, into client chatbox
    ///     "dispatchEventIfClientSendMessage" can be setted
    ///     "description" can be setted.
    ///     
    /// What is ignored:
    ///     FollowLink is ignored.
    ///     
    /// Default behaviour:
    ///     This event must be implemented, there is no default behaviour for this event.
    /// </summary>
    /// <param name="client"></param>
    /// <returns></returns>
    [WebMethod(EnableSession=true)]
    public Client onClientWantsToConnect(Client client)
    {
        
        /*
        //this client will appears in array clients, method onCheckClients after 15 seconds
        //if next check is lower than 1, client will never more call onCheckClients
        client.setNextCheck(15);
        //name of the user or performer
        client.setNickName("nickname");
        //roomID where client will be redirected
        client.setRoomUID("AYNEKCAAKDFHNA");
        //send message to the client that apperas in the chat window
        client.SendRoomMessage("Admin", "FF0000", "User \"" + client.getNickName() + "\" has entered.");
         */
        //accept client into the chat
        client.AcceptConnection();
        //return client info
        return client;
    }

    /// <summary>
    /// Occurs after onClientsWantsToConnect and after client successfully 
    /// handshaked with stream server.
    /// 
    /// What must be done:
    ///     Nothing
    ///     
    /// What could be done:
    ///     "nextCheck" should be setted.
    ///     Chat message can be sended by "SendMessage" function, into client chatbox
    ///     "dispatchEventIfClientSendMessage" can be setted
    ///     "description" can be setted.
    ///     
    /// What is ignored:
    ///     FollowLink is ignored.
    ///     SetRoomUID is ignored.
    ///     
    /// Default behaviour:
    ///     do nothing.
    /// </summary>
    /// <param name="clients"></param>
    /// <returns></returns>
    [WebMethod(EnableSession=true)]
    public Client onClientConnectionWasAccepted(Client client)
    {
        return client;
    }

    /// <summary>
    /// This method is called when client is forcefully rejected from the server
    /// after that onClientDisconnect is called.
    /// </summary>
    /// <param name="client"></param>
    [WebMethod(EnableSession=true)]
    public void onClientConnectionWasRejected(Client client)
    {

    }

    /// <summary>
    /// This method is called when client is disconnected from the server
    /// </summary>
    /// <param name="client"></param>
    [WebMethod(EnableSession=true)]
    public ChatRoom onClientDisconnected(Client client, ChatRoom room)
    {
        return room;
    }

    /// <summary>
    /// This method is called when client wants to stream up.
    /// 
    /// What must be done:
    ///     UserFlag should be setted.
    ///     "setStreamingAccepted" or "setStreamingRejected" should be called
    ///     
    /// What could be done:
    ///     "nextCheck" should be setted.
    ///     Chat message can be sended by "SendMessage" function, into client chatbox
    ///     "dispatchEventIfClientSendMessage" can be setted
    ///     "description" can be setted.
    ///     
    /// What is ignored:
    ///     FollowLink is ignored.
    ///     SetRoomUID is ignored.
    ///     
    /// Default behaviour:
    ///     UserFlag is setted to "default".
    ///     "setStreamingAccepted" is called.
    ///     
    /// </summary>
    /// <param name="client"></param>
    /// <returns></returns>
    [WebMethod(EnableSession=true)]
    public Client onClientWantsToStartStreaming(Client client, Stream stream)
    {
        client.setStreamingAccepted();
        return client;
    }

    /// <summary>
    /// This method appears when client wants to stop streaming up
    /// </summary>
    /// <param name="client"></param>
    [WebMethod(EnableSession=true)]
    public void onClientStoppedStreaming(Client client, Stream stream)
    {

    }

    /// <summary>
    /// This method is called when client wants to attach some stream.
    /// What must be done:
    ///     UserFlag should be setted.
    ///     "setPlayStreamAccepted" or "setPlayStreamRejected" should be called
    ///     
    /// What could be done:
    ///     "nextCheck" should be setted.
    ///     Chat message can be sended by "SendMessage" function, into client chatbox
    ///     "dispatchEventIfClientSendMessage" can be setted
    ///     "description" can be setted.
    ///     
    /// What is ignored:
    ///     FollowLink is ignored.
    ///     SetRoomUID is ignored.
    ///     
    /// Default behaviour:
    ///     "setPlayStreamAccepted" is called.
    /// </summary>
    /// <param name="client"></param>
    /// <returns></returns>
    [WebMethod(EnableSession=true)]
    public Client onClientWantsToPlayStream(Client client, Stream stream)
    {
        client.setPlayStreamAccepted();
        return client;
    }

    /// <summary>
    /// This method is called when client stopped playing stream.
    /// </summary>
    /// <param name="client"></param>
    [WebMethod(EnableSession=true)]
    public void onClientStoppedPlayingStream(Client client, Stream stream)
    {

    }

    /// <summary>
    /// In this method should be reinitialized nextcheck parameter,
    /// some message could be added, or client could be rejected
    /// 
    /// What must be done:
    ///     "nextCheck" should be reinitialized, otherwise client will never be checked again
    ///     
    /// What could be done:
    ///     Chat message can be sended by "SendMessage" function, into client chatbox
    ///     "dispatchEventIfClientSendMessage" can be setted
    ///     "FollowLink" can be called.
    ///     "description" can be setted.
    ///     
    /// What is ignored:
    ///     UserFlag is ignored.
    ///     SetRoomUID is ignored.
    ///     
    /// Default behaviour:
    ///     do nothing.
    /// </summary>
    /// <param name="clients"></param>
    /// <returns></returns>
    [WebMethod(EnableSession=true)]
    public Client[] onCheckClients(Client[] clients)
    {
        foreach (Client client in clients)
        {
            
        }
        return clients;
    }

    


    /// <summary>
    /// This method is called when client sent a text message
    /// and client dispatchEventIfClientSendMessage is true
    /// 
    /// What must be done:
    ///     
    ///     
    /// What could be done:
    ///     Client can be rejected.
    ///     Chat message can be sended by "SendMessage" function, into client chatbox
    ///     "dispatchEventIfClientSendMessage" can be setted
    ///     "description" can be setted.
    ///     
    /// What is ignored:
    ///     UserFlag is ignored.
    ///     SetRoomUID is ignored.
    ///     
    /// Default behaviour:
    ///     Message is sent from client "client" to client with UID "toUID"
    ///     
    /// </summary>
    /// <param name="clientFrom"></param>
    /// <param name="clientTo"></param>
    /// <param name="color"></param>
    /// <param name="message"></param>
    [WebMethod(EnableSession=true)]
    public Client onClientSendMessage(Client clientFrom, string toUID, 
                                      string color, string message)
    {
        clientFrom.SendPrivateMessage(clientFrom.nickname, toUID, color, message);
        return clientFrom;
    }
    
    /// <summary>
    /// This method is called when client press button with universal call
    /// 
    /// What must be done:
    ///     
    ///     
    /// What could be done:
    ///     Client can be rejected.
    ///     Chat message can be sended by "SendMessage" function, into client chatbox
    ///     "dispatchEventIfClientSendMessage" can be setted
    ///     "FollowLink" can be called.
    ///     "description" can be setted.
    ///     
    /// What is ignored:
    ///     UserFlag is ignored.
    ///     SetRoomUID is ignored.
    ///     
    /// Default behaviour:
    ///     do nothing
    ///     
    /// </summary>
    /// <param name="client"></param>
    /// <param name="value"></param>
    /// <returns></returns>
    [WebMethod(EnableSession=true)]
    public Client[] onUniversalCall(Client[] clients, String senderName, String eventName, String value)
    {
        return clients;
    }



    /// <summary>
    /// This method is called when first user enter the room.
    /// 
    /// What must be done:
    ///     Nothing.
    ///     
    /// What could be done:
    ///     "nextCheck" can be setted, otherwise onCheckRooms will never be called by this room.
    ///     "roomName" can be setted.
    ///     "description" can be setted.
    ///     
    /// What is ignored:
    ///     "CloseRoom" is ignored.
    ///     
    /// Default behaviour:
    ///     do nothing
    /// </summary>
    /// <param name="room"></param>
    /// <returns></returns>
    [WebMethod]
    public ChatRoom onChatRoomCreated(ChatRoom room)
    {
        return room;
    }


    /// <summary>
    /// Occurs when at least one chat room wants to be checked.
    /// 
    /// What must be done:
    ///     "nextCheck" should be setted.
    ///     
    /// What could be done:
    ///     "description" can be setted.
    ///     CloseRoom can be called, all clients in the room will be forced to leave room immediatelly.
    ///     
    /// What is ignored:
    ///     changes to "roomName" are ignored.
    ///     
    /// Default behaviour:
    ///     do nothing
    /// </summary>
    /// <param name="rooms"></param>
    /// <returns></returns>
    [WebMethod]
    public ChatRoom[] onCheckChatRooms(ChatRoom[] rooms)
    {
        foreach (ChatRoom room in rooms)
        {
            
        }
        return rooms;
    }



    /// <summary>
    /// Occurs after last client left the room.
    /// </summary>
    /// <param name="room"></param>
    [WebMethod]
    public void onChatRoomClosed(ChatRoom room)
    {

    }
    
}
