﻿/**
 * RadicalChatService
 * Copyright Jaromir Sivic 2009 
 */

using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.Drawing;

/// <summary>
/// Class client
/// </summary>
public class Client
{
    //clients uid
    public string UID = "";
    /// <summary>
    /// Get client unique id - this id was setted up in config.xml or as flashvar
    /// </summary>
    /// <returns></returns>
    public string getUID() { return UID; }
    //client nickname
    public string nickname;
    /// <summary>
    /// Get client nick name
    /// </summary>
    /// <returns></returns>
    public string getNickName() { return nickname; }
    /// <summary>
    /// Set client nick name
    /// </summary>
    /// <param name="value"></param>
    public void setNickName(string value) { nickname = value; }
    //if client is streaming
    public Boolean isStreaming;
    /// <summary>
    /// Return true if client has published at least one stream
    /// </summary>
    /// <returns></returns>
    public Boolean getIsStreaming() { return isStreaming; }
    //if client is connected or not
    public Boolean connected = false;
    //client roomID
    public string roomUID = "";
    /// <summary>
    /// Get room unique ID where client actually is
    /// </summary>
    /// <returns></returns>
    public string getRoomUID() { return roomUID; }
    /// <summary>
    /// Set unique ID of room, where client will be redirected.
    /// This could be done only in onClientWantsToConnect.
    /// </summary>
    /// <param name="value"></param>
    public void setRoomUID(string value) { roomUID = value; }
    //chat text
    public string chatText = "";
    //when client will be checked next time
    public int nextCheck = -1;
    /// <summary>
    /// Return amount of seconds when client will be next time checked by event onCheckClients
    /// </summary>
    /// <returns></returns>
    public int getNextCheck() { return nextCheck; }
    /// <summary>
    /// Set timeout when client will be checked by onCheckClients event
    /// </summary>
    /// <param name="value"></param>
    public void setNextCheck(int value) { nextCheck = value; }
    //when user is redirected to another url
    //public string redirectToURL = "";
    public string commands = "";
    //read only properties like icon, published streams, usersflag
    public string propertiesRO = "";
    //dispatch event when client send next text message
    public Boolean dispatchEventIfClientSendChatMessage = false;
    /// <summary>
    /// Return true if client is dispatching event onClientSendMessage every time
    /// when client send chat message
    /// </summary>
    /// <returns></returns>
    public Boolean getDispatchEventIfClientSendChatMessage() 
    { return dispatchEventIfClientSendChatMessage; }
    /// <summary>
    /// Set if client will dispatch event onClientSendMessage every time
    /// when client send chat text message
    /// </summary>
    /// <param name="value"></param>
    public void setDispatchEventIfClientSendChatMessage(Boolean value)
    { dispatchEventIfClientSendChatMessage = value; }
    //description - is never used by stream server
    public string description = "";
    /// <summary>
    /// Return client description
    /// </summary>
    /// <returns></returns>
    public string getDescription() { return description; }
    /// <summary>
    /// Set client description
    /// </summary>
    /// <param name="value"></param>
    public void setDescription(string value) { description = value; }
    //if user can start streaming or attach stream
    public Boolean StreamAccepted = false;
    //ip address of the client - read only
    public string IP = "";
    /// <summary>
    /// Return client IP address
    /// </summary>
    /// <returns></returns>
    public string getIP() { return IP; }
    //flash version - read only
    public string flashVersion = "";
    /// <summary>
    /// Return version of client's flash player
    /// </summary>
    /// <returns></returns>
    public string getFlashVersion() { return flashVersion; }
    //Flag like performer, user, etc. used in video box.
    public string userFlag = "";
    /// <summary>
    /// Return client userFlag - used only if client can publish stream. UserFlag specify
    /// videopanel where client's stream will be published - see config.xml.
    /// </summary>
    /// <returns></returns>
    public string getUserFlag() { return userFlag; }
    /// <summary>
    /// Set client userFlag - used only if client can publish stream. UserFlag specify
    /// videopanel where client's stream will be published - see config.xml.
    /// </summary>
    /// <param name="value"></param>
    public void setUserFlag(string value) { userFlag = value; }
    /// <summary>
    /// Cookie for sessions
    /// </summary>
    public string cookie = "";

    
    public Client()
	{

	}

    /// <summary>
    /// client will be rejected
    /// </summary>
    public void RejectConnection()
    {
        connected = false;
    }

    /// <summary>
    /// client will be accepted
    /// </summary>
    public void AcceptConnection()
    {
        connected = true;
    }

    /// <summary>
    /// This method will send text message to the client chat window
    /// </summary>
    /// <param name="color"></param>
    /// <param name="userName"></param>
    /// <param name="text"></param>
    public void SendPrivateMessage(string fromUserName, string toUID, string color, string text)
    {
        this.SendIndirectPrivateMessage(this.UID, fromUserName, toUID, color, text);
    }

    /// <summary>
    /// This method will send text message to the client with uid toUID
    /// </summary>
    /// <param name="color"></param>
    /// <param name="userName"></param>
    /// <param name="text"></param>
    public void SendIndirectPrivateMessage(string fromUID, string fromUserName, string toUID, string color, string text)
    {
        //chatText += fromUserName + "~~" + toUID + "~~" + color + "~~" + text + "~!~";
        chatText += "TextMessage" + 
        		    "~~fromClientUID=" + fromUID.Replace('~',';') +
                    "~~fromClientNickName=" + fromUserName.Replace('~',';') + 
        		    "~~toClientUID=" + toUID.Replace('~',';') +
        		    "~~color=" + color.Replace('~',';') +
        		    "~~message=" + text.Replace('~',';') + "~!~";
    }

    /// <summary>
    /// This method will send text message to all clients in chat room
    /// </summary>
    /// <param name="color"></param>
    /// <param name="userName"></param>
    /// <param name="text"></param>
    public void SendRoomMessage(string fromUserName, string color, string text)
    {
        this.SendIndirectPrivateMessage(this.UID, fromUserName, "_ALL", color, text);
    }



    /// <summary>
    /// Add new command to the commands array
    /// </summary>
    /// <param name="commandName"></param>
    /// <param name="parameters"></param>
    private void addCommand(string commandName, string[] parameters)
    {
        //test if command is null
        if(commands==null)
        {
            commands = "";
        }
        //create outparams
        string outparams = "";
        //serialize parameters into outparams
        if (parameters != null)
        {
            foreach (string parameter in parameters)
            {
                outparams += Const.SEPARATOR + parameter;
            }
        }
        //create command
        commands += commandName + outparams + Const.NEWLINE;
    }
    
    /// <summary>
    /// Redirect user to different page
    /// </summary>
    /// <param name="url">url like http://www.mypage.com/nextchat</param>
    /// <param name="target">like _self, _blank</param>
    public void openWebPage(string url, string target)
    {
        addCommand("commandOpenWebPage", new string[] { "URL=" + url, "Target=" + target });
    }

    /// <summary>
    /// Enable streaming - now client can publish the stream
    /// </summary>
    public void setStreamingAccepted()
    {
        StreamAccepted = true;
    }

    /// <summary>
    /// Disable streaming - client will not publish the stream
    /// </summary>
    public void setStreamingRejected()
    {
        StreamAccepted = false;
    }

    /// <summary>
    /// Client can play stream - client can attach live stream
    /// </summary>
    public void setPlayStreamAccepted()
    {
        StreamAccepted = true;
    }

    /// <summary>
    /// Client can´t play stream - client can´t attach live stream
    /// </summary>
    public void setPlayStreamRejected()
    {
        StreamAccepted = false;
    }
}
